/**
 * GenericRecord.java
 * Created on 2005/05/12
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.cache;

import izayoi.Constants;
import izayoi.Util;
import izayoi.exception.CheckSumMismatch;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.Map;

import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentHashMap;
import edu.emory.mathcs.backport.java.util.concurrent.LinkedBlockingQueue;

/**
 * ėṕuR[hv
 * ʂ͂ŎA
 */
public class GenericRecord implements WritableRecord {

    private Map bodyContent = new ConcurrentHashMap(4, 0.75f, 1);
    private String id;
    private Collection key = new LinkedBlockingQueue();
    private long timeStamp;

    /**
     * 񉻂ꂽ琶
     */
    public static final Record parseRecord(String line) throws CheckSumMismatch {
        GenericRecord record;
        try {
            String[] temp = line.split("<>");
            record = new GenericRecord();
            record.timeStamp = Long.parseLong(temp[0]);
            record.id = temp[1];
            for (int i = 2; i < temp.length; i++) {
                String key = temp[i].substring(0, temp[i].indexOf(":"));
                String val = temp[i].substring(temp[i].indexOf(":") + 1);
                record.bodyContent.put(key, val);
                record.key.add(key);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new IllegalArgumentException("wrong format");
        }
        String md5 = Util.calcMD5(record.getBody());
        if (!record.id.equals(md5)) {
            throw new CheckSumMismatch(record.id, md5);
        }
        return record;
    }

    /* ( Javadoc)
    * @see izayoi.cache.CacheRecord#containsField(java.lang.String)
    */
    public final boolean containsField(String name) {
        return key.contains(name);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#getBody()
     */
    public String getBody() {
        StringBuffer sb = new StringBuffer();
        for (Iterator i = key.iterator(); i.hasNext();) {
            String key = (String) i.next();
            sb.append(key + ":" + getField(key));
            if (i.hasNext()) {
                sb.append("<>");
            }
        }
        return sb.toString();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#getField(java.lang.String)
     */
    public final String getField(String name) {
        return (String) bodyContent.get(name);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#getFieldNames()
     */
    public final Collection getFieldNames() {
        return Collections.unmodifiableCollection(key);
    }

    /* ( Javadoc)
     * @see izayoi.cache.Record#getHeader()
     */
    public String getHeader() {
        return getTimeStamp() + "<>" + getID();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#getID()
     */
    public final String getID() {
        if (id == null) {
            setID();
        }
        return id;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#getTimeStamp()
     */
    public final long getTimeStamp() {
        return timeStamp;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheRecord#parseString()
     */
    public final String parseString() {
        return Long.toString(getTimeStamp()) + "<>" + getID() + "<>" + getBody();
    }

    /* ( Javadoc)
     * @see izayoi.cache.WritableRecord#removeField(java.lang.String)
     */
    public final void removeField(String name) {
        if (name.equals(Constants.FIELD_NAME_ID) || name.equals(Constants.FIELD_NAME_TIMESTAMP)) {
            throw new IllegalArgumentException("bad field name :" + name);
        }
        key.remove(name);
        bodyContent.remove(name);
        setID();
    }

    /* ( Javadoc)
     * @see izayoi.cache.WritableRecord#setField(java.lang.String, java.lang.String)
     */
    public final void setField(String name, String value) {
        if (name.equals(Constants.FIELD_NAME_ID) || name.equals(Constants.FIELD_NAME_TIMESTAMP) || !Util.checkFieldName(name)) {
            //\ϖ̂͏R
            throw new IllegalArgumentException("bad field name :" + name);
        }
        if (value == null) {
            return;
        }
        key.add(name);
        bodyContent.put(name, value);
        setID();
    }

    /**
     * ʎqČvZ
     */
    private void setID() {
        id = Util.calcMD5(getBody());
    }

    /* ( Javadoc)
     * @see izayoi.cache.WritableRecord#setTimeStamp(long)
     */
    public final void setTimeStamp(long time) {
        if (time > 9999999999l) {
            time /= 1000;
        }
        timeStamp = time;
    }

    /* ( Javadoc)
     * @see java.lang.Object#toString()
     */
    public final String toString() {
        return parseString();
    }

}
