package izayoi.servlet;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.CacheEntrySorter;
import izayoi.exception.InvalidParameter;
import izayoi.exception.NotImplementedException;
import izayoi.view.DataContainer;
import izayoi.view.TemplateContext;
import izayoi.view.ViewList;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * replace of gateway.cgi
 * @author es
 */
public class ContentsServlet extends BasementServlet {

    private Map typeMap = Util.getTypeNameMap();
    private File stylepath;

    /* ( Javadoc)
     * @see izayoi.servlet.BasementServlet#execute(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String[] arg = parseURL(request.getRequestURI());
        if (arg == null || arg[0] == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }
        String command = arg[0];
        String target = arg[1];

        if (command.equalsIgnoreCase(ACTION_VIEW)) {
            if (target == null) {
                response.sendError(HttpServletResponse.SC_NOT_FOUND);
                return;
            }
            String cmd = request.getParameter(PARAM_COMMAND);
            if (cmd != null && cmd.equals(ACTION_CREATE) && request.getMethod().equalsIgnoreCase("post")) {
                redirect(request, response, target);
            } else {
                getView(request, response, target);
            }
        } else if (command.equalsIgnoreCase(ACTION_PREF)) {
            if (request.getMethod().equalsIgnoreCase("post")) {
                setPref(request, response);
            } else {
                getPerf(request, response);
            }
        } else {
            throw new NotImplementedException(command);
        }
    }

    /**
     * @param request
     * @param response
     */
    private void setPref(HttpServletRequest request, HttpServletResponse response) throws IOException {
		String style = request.getParameter(PARAM_STYLE);
		if(!style.matches("(\\w|-)+")) {
			style = ".";
		}
		String sortlist = request.getParameter(PARAM_SORT_LIST);
		String orderlist = request.getParameter(PARAM_ORDER_LIST);
		String orderthread = request.getParameter(PARAM_ORDER_THREAD);

		Cookie[] c = new Cookie[] {
			new Cookie(PARAM_STYLE, style),
			new Cookie(PARAM_SORT_LIST, sortlist),
			new Cookie(PARAM_ORDER_LIST, orderlist),
			new Cookie(PARAM_ORDER_THREAD, orderthread)
		};
		for(int i=0; i < c.length; i++) {
			c[i].setMaxAge(60 * 60 * 24 * 30);
			c[i].setPath("/");
			response.addCookie(c[i]);
		}

        // TODO [g߂~߂
        response.sendRedirect("/" + Util.getTypeExecutor(Constants.TYPE_LIST));
    }

    /**
     * @param request
     * @param response
     * @param target
     */
    private void redirect(HttpServletRequest request, HttpServletResponse response, String type) throws InvalidParameter, IOException {
        String basename = request.getParameter(PARAM_NAME);
        if (basename == null || basename.equals("")) {
            throw new InvalidParameter("O", "l܂");
        }
        String exec = Util.getTypeExecutor(type);
        if (exec == null) {
            throw new InvalidParameter("", type);
        }
		// TODO [g߂~߂
        response.sendRedirect("/" + exec + "/" + Util.urlEncode(Util.escape(basename)));
    }

    /**
     * @param request
     * @param response
     */
    private void getPerf(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        TemplateContext ctx = new TemplateContext("pref.txt", request, response);

        File[] f = stylepath.listFiles(new FileFilter() {
            public boolean accept(File arg0) {
                return arg0.isDirectory() && arg0.getName().matches("(\\w|-)+");
            }
        });
        List l = new ArrayList();
        for (int i = 0; i < f.length; i++) {
            l.add(f[i].getName());
        }
        ctx.put("records", l.toArray());
        ctx.finish();
    }

    /**
     * LbVꗗ\
     * @param request
     * @param response
     * @param target Ώۂ̃^Cv(thread/list/note)
     */
    private void getView(HttpServletRequest request, HttpServletResponse response, String target)
        throws IOException, ServletException {
        if (!typeMap.containsKey(target)) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

		//\[g擾
		Map reqMap = request.getParameterMap();
		String sort = (String) request.getAttribute(PARAM_SORT_LIST);
		if(reqMap.containsKey(PARAM_SORT)) {
			sort = request.getParameter(PARAM_SORT);
		}
		String order = (String) request.getAttribute(PARAM_ORDER_LIST);
		if(reqMap.containsKey(PARAM_ORDER)) {
			order = request.getParameter(PARAM_ORDER);
		}
		order = (order == null) ? "" : order;
		sort = (sort == null) ? "" : sort;
        CacheEntrySorter sorter = null;
        if (sort.equals(Constants.SORT_BYCOUNT)) {
            sorter = new CacheEntrySorter.ByCount();
        } else if (sort.equals(Constants.SORT_BYNAME)) {
            sorter = new CacheEntrySorter.ByName();
        } else if (sort.equals(Constants.SORT_BYLAST)) {
            sorter = new CacheEntrySorter.ByLastMod();
        } else if (sort.equals(Constants.SORT_BYSIZE)) {
            sorter = new CacheEntrySorter.BySize();
        }
        if (sorter != null) {
            sorter.setOrder(!order.equals(Constants.SORT_DESC));
        }

		//LbV擾
        DataContainer[] data = ViewList.parseContent(cache.getEntris(target, sorter));

        TemplateContext ctx = new TemplateContext("contentslist.txt", request, response);
        ctx.setRecord(data);
        ctx.setName((String) typeMap.get(target) + "ꗗ");
        ctx.finish();
    }

    /* ( Javadoc)
     * @see javax.servlet.Servlet#destroy()
     */
    public void destroy() {
        super.destroy();
    }

    /* ( Javadoc)
     * @see izayoi.servlet.BasementServlet#initalize(javax.servlet.ServletConfig)
     */
    protected void initalize(ServletConfig config) throws ServletException {
        stylepath = new File(config.getServletContext().getRealPath(conf.getParameter("stylePath")));
    }

}
