/**
 * ViewThread.java
 * Created on 2005/06/08
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.view;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.Record;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * threadñev[gɗRei
 * @author es
 */
public class ViewThread implements DataContainer {

    private static final Pattern ancher = Pattern.compile("&gt;&gt;([0-9a-f]{8})");
    private static final Pattern http = Pattern.compile("http\\:\\/\\/[-_.!~*'\\(\\)a-zA-Z0-9;/?:@&=+$,%#]+");
    private static final Pattern link =
        Pattern.compile("\\[\\[((\\/([thread|note|list]+)\\/)?([\\S&&[^\\/]]+)(/([0-9a-f]{8}))?)\\]\\]");

    private String attach;
    private String body;
    private Date date;
    private String id;
    private String key;
    private String mail;
    private String name;
    private String size;
    private long time;

    /**
     * Record[]炩ev[gp̃Reizɔz
     * @param record
     * @param type AvP[V("[[../../../]]N̂ɕKv")
     * @return
     */
    public static DataContainer[] parseEntry(Record[] record, String type) {
        if (record == null) {
            record = new Record[0];
        }
		DataContainer[] v = new DataContainer[record.length];
        for (int i = 0; i < record.length; i++) {
            v[i] = ViewThread.parseEntry(record[i], type);
        }
        return v;
    }

    /**
     * Recordev[gp̃Rei
     * @param record
     * @param type AvP[V("[[../../../]]N̂ɕKv")
     * @return
     */
    public static DataContainer parseEntry(Record record, String type) {
        ViewThread v = new ViewThread();
        v.time = record.getTimeStamp();
        v.date = new Date(v.time * 1000);
        v.id = record.getID();
        v.key = v.id.substring(0, 8);
        v.name = record.getField(Constants.FIELD_NAME_NAME);
        v.mail = record.getField(Constants.FIELD_NAME_MAIL);

        if (record.containsField(Constants.FIELD_NAME_ATTACH)) {
            v.attach = v.time + "." + record.getField(Constants.FIELD_NAME_SUFFIX);
            v.size = Util.parseByteString((long)(record.getField(Constants.FIELD_NAME_ATTACH).length() * 3 / 4));
        }

        //{̎N
        String body = record.containsField(Constants.FIELD_NAME_BODY) ? record.getField(Constants.FIELD_NAME_BODY) : "";
        body = ancher.matcher(body).replaceAll("<a href='#$1'>$0</a>");
        body = http.matcher(body).replaceAll("<a href='$0'>$0</a>");
        Matcher m = link.matcher(body);
        StringBuffer sb = new StringBuffer();
        for (boolean b = m.find(); b;) {
            String linktype = m.group(3);
            String linkname = m.group(4);
            String linkid = m.group(6);
            if (linktype == null) {
                linktype = type;
            }
            if (linkid == null) {
                linkid = "";
            } else {
                linkid = "#" + linkid;
            }
            linkname = Util.urlEncode(linkname);
            String tg = "[[<a href='/" + linktype + ".cgi/" + linkname + linkid + "'>$1</a>]]";
            m = m.appendReplacement(sb, tg);
            b = m.find();
        }
        m.appendTail(sb);

        v.body = sb.toString();
        return v;
    }

    /**
     * @return
     */
    public String getAttach() {
        return attach;
    }

    /**
     * @return
     */
    public String getBody() {
        return body;
    }

    /**
     * @return
     */
    public String getDate() {
        return SimpleDateFormat.getDateTimeInstance().format(date);
    }

    /**
     * @return
     */
    public String getId() {
        return id;
    }

    /**
     * @return
     */
    public String getKey() {
        return key;
    }

    /**
     * @return
     */
    public String getMail() {
        return mail;
    }

    /**
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * @return
     */
    public String getSize() {
        return size;
    }

    /**
     * @return
     */
    public long getTime() {
        return time;
    }

}
